#import <Cocoa/Cocoa.h>
#import "RIRipEngine.h"
#import "RIRipMode.h"
#import "DKDVDStructure.h"
#import "RIDiscPatch.h"


@interface RIRipController : NSObject 
{
    NSString				*deviceName;
    NSString				*mountPath;
    NSString				*moviesFolder;
	NSString				*originalMoviesFolder;
    NSString				*title;
    NSString				*discTitle;
    NSString				*discidentTitle;
    NSString				*fingerprint;
    NSMutableDictionary		*status;
    BOOL					 wasCancelled;
    BOOL					 shouldEjectWhenFinished;

    uint64_t				 capacity;
    uint64_t				 bytesToCopy;
    uint64_t				 bytesCopied;
    FSRef					*output;
    NSOperationQueue		*ripOperationQueue;
    NSOperationQueue		*writebackOperationQueue;
    NSMutableDictionary		*handlesForWriting;
    NSMutableArray			*statisticsWindow;

    NSDate					*firstRead;
    NSDate					*lastReadInWindow;
    uint64_t				 bytesInWindow;

	// nsv
	RIRipMode				 ripMode;
	BOOL					 isUDF;
	id						 delegate;
	//RIDiscPatchBuilder	*discPatchBuilder;
	uint64_t				 queueSize;
	RIDiscPatch*			 discPatch;
	
	// FILTERED RIPPING
	BOOL					filterOptimizeSkipLongMenus;
	BOOL					filterOptimizeSkipBranchLanguage;
	BOOL					filterOptimizeSkipBranchDisplayFormat;
	NSArray					*filterTitles;
	NSMutableDictionary		*filterTitleSetFileDictionary;
}

- (id) initWithURL:(NSURL*)url fingerprint:(NSString*)fingerprint discTitle:(NSString*)discTitle;
- (id) initWithDeviceName:(NSString*)deviceName fingerprint:(NSString*)fingerprint discTitle:(NSString*)discTitle;
- (id)initWithDeviceName:(NSString*)_deviceName mountPoint:(NSString *)_mountPoint fingerprint:(NSString*)_fingerprint discTitle:(NSString*)_discTitle;
- (void) analyzeWithEngine:(Class)engine;
- (void) analyzeDeepWithEngine:(Class)engine;
- (void) startWithEngine:(Class)engine filterFor:(NSArray *)titles;
- (void) startWithEngine2:(Class)engine;
- (void) startWithEngine:(Class)engine;
- (void) cancel;
- (BOOL) isFiltered;

// nsv
@property (assign) id				delegate;
@property (assign) uint64_t			queueSize;
@property (retain) RIDiscPatch*		discPatch;

@property (readonly) NSString*		deviceName;
@property (retain) NSString*		moviesFolder;
@property (retain) NSString*		originalMoviesFolder;
@property (retain) NSString*		mountPath;
@property (retain) NSString*		title;
@property (readonly) NSString*		discTitle;
@property (readonly) NSString*		fingerprint;
@property (readonly) NSDictionary*	status;
@property (assign) uint64_t			capacity;
@property (readonly) NSString*		dvdMediaName;
@property (assign) BOOL				shouldEjectWhenFinished;
@property (readonly) BOOL			wasCancelled;

// FILTERED RIPPING
@property (assign) BOOL				filterOptimizeSkipLongMenus;
@property (assign) BOOL				filterOptimizeSkipBranchLanguage;
@property (assign) BOOL				filterOptimizeSkipBranchDisplayFormat;


@end

extern NSString* const RIStatusReasonForFailureKey;
extern NSString* const RIStatusBytesPerSecondKey;
extern NSString* const RIStatusEstimatedTimeKey;
extern NSString* const RIStatusPercentageCompleteKey;
extern NSString* const RIStatusCurrentFileKey;
extern NSString* const RIStatusStateKey;
extern NSString* const RIStatusBytesCopiedKey;
extern NSString* const RIStatusBytesToCopyKey;
extern NSString* const RIStatusDVDMediaFilePathKey;
extern NSString* const RIStatusPlanOfAttackKey;
extern NSString* const RIStatusQueueSizeKey;

extern NSString* const RIStatusStateReady;
extern NSString* const RIStatusStatePreparing;
extern NSString* const RIStatusStateRipping;
extern NSString* const RIStatusStateZeroingSpace;
extern NSString* const RIStatusStateFinished;
extern NSString* const RIStatusStateCancelled;
extern NSString* const RIStatusStateFailed;
extern NSString* const RIStatusStateWaiting;

extern NSString* const RIRipControllerException;









